#pragma once

#include "clap.h"

#include "factory/draft/plugin-invalidation.h"
#include "factory/draft/plugin-state-converter.h"

#include "ext/draft/background-activation.h"
#include "ext/draft/background-progress.h"
#include "ext/draft/background-state-context.h"
#include "ext/draft/extensible-audio-ports.h"
#include "ext/draft/flush-events.h"
#include "ext/draft/gain-adjustment-metering.h"
#include "ext/draft/mini-curve-display.h"
#include "ext/draft/octave-number.h"
#include "ext/draft/params-origin.h"
#include "ext/draft/param-hovered.h"
#include "ext/draft/project-location.h"
#include "ext/draft/resource-directory.h"
#include "ext/draft/scratch-memory.h"
#include "ext/draft/transport-control.h"
#include "ext/draft/triggers.h"
#include "ext/draft/tuning.h"
#include "ext/draft/undo.h"
#include "ext/draft/webview.h"
